@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns, OverloadedStrings, ExistentialQuantification #-}
+{-# LANGUAGE BangPatterns, OverloadedStrings, TemplateHaskell, ExistentialQuantification #-}
 
 {-|
 Module      : Client.Commands
@@ -25,44 +25,45 @@ module Client.Commands
   , commandsList
   ) where
 
-import           Client.Commands.Arguments.Spec
-import           Client.Commands.Arguments.Parser
-import           Client.Commands.Exec
-import           Client.Commands.Interpolation
-import           Client.Commands.Recognizer
-import           Client.Commands.WordCompletion
-import           Client.Configuration
-import           Client.State
-import           Client.State.Extensions
-import           Client.State.Focus
-import           Client.State.Network
-import           Client.State.Window
-import           Control.Applicative
-import           Control.Exception (displayException, try)
-import           Control.Lens
-import           Control.Monad
-import           Data.Foldable
-import           Data.Text (Text)
-import qualified Data.Text as Text
-import           Data.Time (getZonedTime)
-import           Irc.Commands
-import           Irc.Identifier
-import           Irc.RawIrcMsg
-import           Irc.Message
-import           RtsStats (getStats)
-import           System.Process
+import Client.Commands.Arguments.Parser (parse)
+import Client.Commands.Arguments.Spec (optionalArg, optionalNumberArg, remainingArg, simpleToken)
+import Client.Commands.Docs (clientDocs, cmdDoc)
+import Client.Commands.Exec
+import Client.Commands.Interpolation (resolveMacroExpansions, Macro(Macro), MacroSpec(MacroSpec))
+import Client.Commands.Recognizer (fromCommands, keys, recognize, Recognition(Exact), Recognizer)
+import Client.Commands.WordCompletion (caseText, plainWordCompleteMode, wordComplete)
+import Client.Configuration
+import Client.State
+import Client.State.Extensions (clientCommandExtension, clientStartExtensions)
+import Client.State.Focus
+import Client.State.Network (csNick, isChannelIdentifier, sendMsg)
+import Client.State.Url
+import Control.Applicative (liftA2, (<|>))
+import Control.Exception (displayException, try)
+import Control.Lens
+import Control.Monad (guard, foldM)
+import Data.Foldable (foldl', toList)
+import Data.Text (Text)
+import Data.Text qualified as Text
+import Data.Time (getZonedTime)
+import Irc.Commands (ircPrivmsg)
+import Irc.Identifier (idText)
+import Irc.Message (IrcMsg(Privmsg))
+import Irc.RawIrcMsg (parseRawIrcMsg)
+import RtsStats (getStats)
+import System.Process.Typed (proc, runProcess_)
 
-import           Client.Commands.Channel (channelCommands)
-import           Client.Commands.Certificate (newCertificateCommand)
-import           Client.Commands.Chat (chatCommands, chatCommand', executeChat)
-import           Client.Commands.Connection (connectionCommands)
-import           Client.Commands.Operator (operatorCommands)
-import           Client.Commands.Queries (queryCommands)
-import           Client.Commands.Toggles (togglesCommands)
-import           Client.Commands.Window (windowCommands)
-import           Client.Commands.ZNC (zncCommands)
-import           Client.Commands.TabCompletion
-import           Client.Commands.Types
+import Client.Commands.Certificate (newCertificateCommand)
+import Client.Commands.Channel (channelCommands)
+import Client.Commands.Chat (chatCommands, chatCommand', executeChat)
+import Client.Commands.Connection (connectionCommands)
+import Client.Commands.Operator (operatorCommands)
+import Client.Commands.Queries (queryCommands)
+import Client.Commands.TabCompletion
+import Client.Commands.Toggles (togglesCommands)
+import Client.Commands.Types
+import Client.Commands.Window (windowCommands)
+import Client.Commands.ZNC (zncCommands)
 
 -- | Interpret the given chat message or command. Leading @/@ indicates a
 -- command. Otherwise if a channel or user query is focused a chat message will
@@ -227,93 +228,55 @@ commandsList =
   [ Command
       (pure "exit")
       (pure ())
-      "Exit the client immediately.\n"
+      $(clientDocs `cmdDoc` "exit")
     $ ClientCommand cmdExit noClientTab
 
   , Command
       (pure "reload")
       (optionalArg (simpleToken "[filename]"))
-      "Reload the client configuration file.\n\
-      \\n\
-      \If \^Bfilename\^B is provided it will be used to reload.\n\
-      \Otherwise the previously loaded configuration file will be reloaded.\n"
+      $(clientDocs `cmdDoc` "reload")
     $ ClientCommand cmdReload tabReload
 
   , Command
       (pure "extension")
       (liftA2 (,) (simpleToken "extension") (remainingArg "arguments"))
-      "Calls the process_command callback of the given extension.\n\
-      \\n\
-      \\^Bextension\^B should be the name of the loaded extension.\n"
+      $(clientDocs `cmdDoc` "extension")
     $ ClientCommand cmdExtension simpleClientTab
 
   , Command
       (pure "palette")
       (pure ())
-      "Show the current palette settings and a color chart to help pick new colors.\n"
+      $(clientDocs `cmdDoc` "palette")
     $ ClientCommand cmdPalette noClientTab
 
   , Command
       (pure "digraphs")
       (pure ())
-      "\^BDescription:\^B\n\
-      \\n\
-      \    Show the table of digraphs. A digraph is a pair of characters\n\
-      \    can be used together to represent an uncommon character. Type\n\
-      \    the two-character digraph corresponding to the desired output\n\
-      \    character and then press M-k (default binding).\n\
-      \\n\
-      \    Note that the digraphs list is searchable with /grep.\n\
-      \\n\
-      \\^BSee also:\^B grep\n"
+      $(clientDocs `cmdDoc` "digraphs")
     $ ClientCommand cmdDigraphs noClientTab
 
   , Command
       (pure "keymap")
       (pure ())
-      "Show the key binding map.\n\
-      \\n\
-      \Key bindings can be changed in configuration file. See `glirc --config-format`.\n"
+      $(clientDocs `cmdDoc` "keymap")
     $ ClientCommand cmdKeyMap noClientTab
 
   , Command
       (pure "rtsstats")
       (pure ())
-      "Show the GHC RTS statistics.\n"
+      $(clientDocs `cmdDoc` "rtsstats")
     $ ClientCommand cmdRtsStats noClientTab
 
   , Command
       (pure "exec")
       (remainingArg "arguments")
-      "Execute a command synchnonously sending the to a configuration destination.\n\
-      \\n\
-      \\^Barguments\^B: [-n[network]] [-c[channel]] [-i input] command [arguments...]\n\
-      \\n\
-      \When \^Binput\^B is specified it is sent to the stdin.\n\
-      \\n\
-      \When neither \^Bnetwork\^B nor \^Bchannel\^B are specified output goes to client window (*).\n\
-      \When \^Bnetwork\^B is specified output is sent as raw IRC traffic to the network.\n\
-      \When \^Bchannel\^B is specified output is sent as chat to the given channel on the current network.\n\
-      \When \^Bnetwork\^B and \^Bchannel\^B are specified output is sent as chat to the given channel on the given network.\n\
-      \\n\
-      \\^Barguments\^B is divided on spaces into words before being processed\
-      \ by getopt. Use Haskell string literal syntax to create arguments with\
-      \ escaped characters and spaces inside.\n\
-      \\n"
+      $(clientDocs `cmdDoc` "exec")
     $ ClientCommand cmdExec simpleClientTab
 
   , Command
       (pure "url")
       optionalNumberArg
-      "Open a URL seen in chat.\n\
-      \\n\
-      \The URL is opened using the executable configured under \^Burl-opener\^B.\n\
-      \\n\
-      \When this command is active in the textbox, chat messages are filtered to\
-      \ only show ones with URLs.\n\
-      \\n\
-      \When \^Bnumber\^B is omitted it defaults to \^B1\^B. The number selects the\
-      \ URL to open counting back from the most recent.\n"
+      $(clientDocs `cmdDoc` "url")
     $ ClientCommand cmdUrl noClientTab
 
   , newCertificateCommand
@@ -321,10 +284,7 @@ commandsList =
   , Command
       (pure "help")
       (optionalArg (simpleToken "[command]"))
-      "Show command documentation.\n\
-      \\n\
-      \When \^Bcommand\^B is omitted a list of all commands is displayed.\n\
-      \When \^Bcommand\^B is specified detailed help for that command is shown.\n"
+      $(clientDocs `cmdDoc` "help")
     $ ClientCommand cmdHelp tabHelp
 
   ------------------------------------------------------------------------
@@ -507,13 +467,8 @@ cmdUrl st arg =
     Nothing     -> commandFailureMsg "url-opener not configured" st
     Just opener -> doUrlOpen opener (maybe 0 (subtract 1) arg)
   where
-    focus = view clientFocus st
-
-    urls = toListOf ( clientWindows . ix focus . winMessages . each . wlText
-                    . folding urlMatches) st
-
     doUrlOpen opener n =
-      case preview (ix n) urls of
+      case preview (ix n) (map fst (urlList st)) of
         Just url -> openUrl opener (Text.unpack url) st
         Nothing  -> commandFailureMsg "bad url number" st
 
@@ -521,7 +476,7 @@ openUrl :: UrlOpener -> String -> ClientState -> IO CommandResult
 openUrl (UrlOpener opener args) url st =
   do let argStr (UrlArgLiteral str) = str
          argStr UrlArgUrl           = url
-     res <- try (callProcess opener (map argStr args))
+     res <- try (runProcess_ (proc opener (map argStr args)))
      case res of
        Left e  -> commandFailureMsg (Text.pack (displayException (e :: IOError))) st
        Right{} -> commandSuccess st
