@@ -46,7 +46,9 @@ module Client.Configuration
   , configShowPing
   , configJumpModifier
   , configDigraphs
+  , configNotifications
 
+  , configNetworkPalette
   , extensionPath
   , extensionRtldFlags
   , extensionArgs
@@ -70,37 +72,37 @@ module Client.Configuration
   , UrlArgument(..)
   ) where
 
-import           Client.Commands.Interpolation
-import           Client.Commands.Recognizer
-import           Client.Configuration.Colors
-import           Client.Configuration.Macros (macroMapSpec)
-import           Client.Configuration.ServerSettings
-import           Client.EventLoop.Actions
-import           Client.Image.Palette
-import           Config
-import           Config.Macro
-import           Config.Schema
-import           Control.Exception
-import           Control.Lens                        hiding (List)
-import           Data.Foldable                       (foldl', toList)
-import           Data.HashMap.Strict                 (HashMap)
-import qualified Data.HashMap.Strict                 as HashMap
-import qualified Data.List.NonEmpty                  as NonEmpty
-import           Data.Map                            (Map)
-import qualified Data.Map                            as Map
-import           Data.Maybe
-import           Data.Monoid                         (Endo(..))
-import           Data.Text                           (Text)
-import qualified Data.Text                           as Text
-import qualified Data.Vector                         as Vector
-import           Digraphs (Digraph(..))
-import           Graphics.Vty.Input.Events (Modifier(..), Key(..))
-import           Graphics.Vty.Attributes             (Attr)
-import           Irc.Identifier                      (Identifier)
-import           System.Directory
-import           System.FilePath
-import           System.Posix.DynamicLinker          (RTLDFlags(..))
-import           System.IO.Error
+import Client.Commands.Interpolation (Macro)
+import Client.Commands.Recognizer (Recognizer)
+import Client.Configuration.Colors (attrSpec)
+import Client.Configuration.Macros (macroMapSpec)
+import Client.Configuration.Notifications (NotifyWith, notifySpec, notifyWithDefault)
+import Client.Configuration.ServerSettings
+import Client.EventLoop.Actions
+import Client.Image.Palette
+import Config
+import Config.Macro
+import Config.Schema
+import Control.Exception
+import Control.Lens hiding (List)
+import Data.Foldable (foldl', toList)
+import Data.HashMap.Strict (HashMap)
+import Data.HashMap.Strict qualified as HashMap
+import Data.List.NonEmpty qualified as NonEmpty
+import Data.Map  (Map)
+import Data.Map qualified as Map
+import Data.Maybe (fromMaybe, isNothing)
+import Data.Monoid (Endo(..))
+import Data.Text (Text)
+import Data.Text qualified as Text
+import Data.Vector qualified as Vector
+import Digraphs (Digraph(..))
+import Graphics.Vty.Input.Events (Modifier(..), Key(..))
+import Irc.Identifier (Identifier, mkId)
+import System.Directory ( getHomeDirectory, getXdgDirectory, XdgDirectory(XdgConfig))
+import System.FilePath ((</>), isAbsolute, joinPath, splitDirectories, takeDirectory)
+import System.IO.Error (ioeGetFileName, isDoesNotExistError)
+import System.Posix.DynamicLinker (RTLDFlags(..))
 
 -- | Top-level client configuration information. When connecting to a
 -- server configuration from '_configServers' is used where possible,
@@ -125,6 +127,7 @@ data Configuration = Configuration
   , _configShowPing        :: Bool -- ^ visibility of ping time
   , _configJumpModifier    :: [Modifier] -- ^ Modifier used for jumping windows
   , _configDigraphs        :: Map Digraph Text -- ^ Extra digraphs
+  , _configNotifications   :: NotifyWith
   }
   deriving Show
 
@@ -294,6 +297,8 @@ configurationSpec = sectionsSpec "config-file" $
                                "Initial setting for visibility of ping times"
      _configDigraphs        <- sec' mempty "extra-digraphs" (Map.fromList <$> listSpec digraphSpec)
                                "Extra digraphs"
+     _configNotifications   <- sec' notifyWithDefault "notifications" notifySpec
+                               "Whether and how to show desktop notifications"
      return (\def ->
              let _configDefaults = snd ssDefUpdate def
                  _configServers  = buildServerMap _configDefaults ssUpdates
@@ -403,29 +408,18 @@ paletteSpec = sectionsSpec "palette" $
     nickColorsSpec = set palNicks . Vector.fromList . NonEmpty.toList
                  <$> nonemptySpec attrSpec
 
-    modeColorsSpec :: Lens' Palette (HashMap Char Attr) -> ValueSpec (Palette -> Palette)
-    modeColorsSpec l
-      = fmap (set l)
-      $ customSpec "modes" (assocSpec attrSpec)
-      $ fmap HashMap.fromList
-      . traverse (\(mode, attr) ->
-          case Text.unpack mode of
-            [m] -> Right (m, attr)
-            _   -> Left "expected single letter")
+    idOverridesSpec = set palIdOverride . HashMap.fromList . concat <$> listSpec idOverrideSpec
+    idOverrideSpec =
+      sectionsSpec "ids-to-attr" $
+      do ids <- reqSection' "ids" (oneOrList stringSpec) "One or more identifiers that this override applies to."
+         attr <- reqSection' "color" attrSpec "The style to use."
+         pure [(mkId $ Text.pack id', attr) | id' <- ids]
 
     fields :: [SectionsSpec (Maybe (Palette -> Palette))]
-    fields = optSection' "nick-colors" nickColorsSpec
-             "Colors used to highlight nicknames"
-
-           : optSection' "cmodes" (modeColorsSpec palCModes)
-             "Colors used to highlight channel modes"
-
-           : optSection' "umodes" (modeColorsSpec palUModes)
-             "Colors used to highlight user modes"
-
-           : optSection' "snomask" (modeColorsSpec palSnomask)
-             "Colors used to highlight server notice mask"
-
+    fields = optSection' "identifier-colors" nickColorsSpec
+             "Colors used to highlight identifiers (nicks, channel names)."
+           : optSection' "identifier-overrides" idOverridesSpec
+             "Colors used to highlight specific identifiers (nicks, channel names)."
            : [ optSection' lbl (set l <$> attrSpec) "" | (lbl, Lens l) <- paletteMap ]
 
 extensionSpec :: ValueSpec ExtensionConfiguration
@@ -529,3 +523,10 @@ resolveFilePath fpc path
   | isAbsolute path                   = path
   | "~":rest <- splitDirectories path = joinPath (fpHome fpc : rest)
   | otherwise                         = fpBase fpc </> path
+
+-- | Returns a NetworkPalette for the given network name.
+configNetworkPalette :: Text -> Configuration -> NetworkPalette
+configNetworkPalette net cfg = unifyNetworkPalette palDefault $ fromMaybe defaultNetworkPalette palNet
+  where
+    palNet = view ssPalette <$> view (configServers . at net) cfg
+    palDefault = view (configDefaults. ssPalette) cfg
