@@ -1,4 +1,4 @@
-{-# Language BangPatterns, OverloadedStrings, NondecreasingIndentation #-}
+{-# Language BangPatterns, OverloadedStrings, NondecreasingIndentation, PatternSynonyms #-}
 
 {-|
 Module      : Client.EventLoop
@@ -17,51 +17,53 @@ module Client.EventLoop
   , ClientEvent(..)
   ) where
 
-import           Client.CApi (ThreadEntry, popTimer)
-import           Client.Commands
-import           Client.Configuration (configJumpModifier, configKeyMap, configWindowNames, configDigraphs)
-import           Client.Configuration.ServerSettings
-import           Client.EventLoop.Actions
-import           Client.EventLoop.Errors (exceptionToLines)
-import           Client.EventLoop.Network (clientResponse)
-import           Client.Hook
-import           Client.Image
-import           Client.Image.Layout (scrollAmount)
-import           Client.Image.StatusLine (clientTitle)
-import           Client.Log
-import           Client.Message
-import           Client.Network.Async
-import           Client.State
-import qualified Client.State.EditBox as Edit
-import           Client.State.Extensions
-import           Client.State.Focus
-import           Client.State.Network
-import           Control.Concurrent.STM
-import           Control.Exception
-import           Control.Lens
-import           Control.Monad
-import           Data.ByteString (ByteString)
-import           Data.Char (isSpace)
-import           Data.Foldable
-import           Data.Traversable
-import           Data.List
-import           Data.List.NonEmpty (NonEmpty, nonEmpty)
-import           Data.Maybe
-import           Data.Ord
-import           Data.Text (Text)
-import qualified Data.HashMap.Strict as HashMap
-import qualified Data.Text as Text
-import qualified Data.Text.Encoding as Text
-import qualified Data.Text.Encoding.Error as Text
-import           Data.Time
-import           GHC.IO.Exception (IOErrorType(..), ioe_type)
-import           Graphics.Vty
-import           Graphics.Vty.Input.Events
-import           Irc.Message
-import           Irc.Codes
-import           Irc.RawIrcMsg
-import           LensUtils
-import           Hookup (ConnectionFailure(..))
+import Client.CApi (ThreadEntry, popTimer)
+import Client.Commands (CommandResult(..), execute, executeUserCommand, tabCompletion)
+import Client.Configuration (configJumpModifier, configKeyMap, configWindowNames, configDigraphs, configNotifications)
+import Client.Configuration.Notifications (notifyCmd)
+import Client.Configuration.ServerSettings ( ssReconnectAttempts )
+import Client.EventLoop.Actions (keyToAction, Action(..))
+import Client.EventLoop.Errors (exceptionToLines)
+import Client.EventLoop.Network (clientResponse)
+import Client.Hook (applyMessageHooks, messageHookStateful)
+import Client.Image (clientPicture)
+import Client.Image.Layout (scrollAmount)
+import Client.Image.StatusLine (clientTitle)
+import Client.Log ( writeLogLine )
+import Client.Message
+import Client.Network.Async
+import Client.State
+import Client.State.EditBox qualified as Edit
+import Client.State.Extensions
+import Client.State.Focus (Subfocus(FocusMessages))
+import Client.State.Network
+import Control.Concurrent.STM
+import Control.Exception (SomeException, Exception(fromException), catch)
+import Control.Lens
+import Control.Monad (when, MonadPlus(mplus), foldM, unless, void)
+import Data.ByteString (ByteString)
+import Data.Char (isSpace)
+import Data.Foldable (Foldable(foldl'), find, asum, traverse_)
+import Data.HashMap.Strict qualified as HashMap
+import Data.List (partition)
+import Data.List.NonEmpty (NonEmpty, nonEmpty)
+import Data.Maybe (fromMaybe)
+import Data.Ord (comparing)
+import Data.Text (Text)
+import Data.Text qualified as Text
+import Data.Text.Encoding qualified as Text
+import Data.Text.Encoding.Error qualified as Text
+import Data.Time
+import Data.Time.Format.ISO8601 (formatParseM, iso8601Format)
+import Data.Traversable (for)
+import GHC.IO.Exception (IOErrorType(..), ioe_type)
+import Graphics.Vty
+import Hookup (ConnectionFailure(..))
+import Irc.Codes (pattern RPL_STARTTLS)
+import Irc.Message (IrcMsg(Reply, Notice), cookIrcMsg, msgTarget)
+import Irc.RawIrcMsg (RawIrcMsg, TagEntry(..), asUtf8, msgTags, parseRawIrcMsg)
+import LensUtils (setStrict)
+import System.Process.Typed (startProcess, setStdin, setStdout, setStderr, nullStream)
 
 
 -- | Sum of the five possible event types the event loop handles
@@ -83,7 +85,7 @@ getEvent vty st =
   do timer <- prepareTimer
      atomically (asum [timer, vtyEvent, networkEvents, threadJoin])
   where
-    vtyEvent = VtyEvent <$> readTChan (_eventChannel (inputIface vty))
+    vtyEvent = VtyEvent <$> readTChan (eventChannel (inputIface vty))
 
     networkEvents =
       do xs <- for (HashMap.toList (view clientConnections st)) $ \(network, conn) ->
@@ -135,6 +137,7 @@ earliestEvent st = earliest2 networkEvent extensionEvent
 eventLoop :: Vty -> ClientState -> IO ()
 eventLoop vty st =
   do when (view clientBell st) (beep vty)
+     processNotifications st
      processLogEntries st
 
      let (pic, st') = clientPicture (clientTick st)
@@ -151,7 +154,7 @@ eventLoop vty st =
          eventLoop vty =<< doTimerEvent networkId action st'
        VtyEvent (InputEvent vtyEvent) ->
          traverse_ (eventLoop vty) =<< doVtyEvent vty vtyEvent st'
-       VtyEvent ResumeAfterSignal ->
+       VtyEvent ResumeAfterInterrupt ->
          eventLoop vty =<< updateTerminalSize vty st
        NetworkEvents networkEvents ->
          eventLoop vty =<< foldM doNetworkEvent st' networkEvents
@@ -175,6 +178,20 @@ processLogEntries :: ClientState -> IO ()
 processLogEntries =
   traverse_ writeLogLine . reverse . view clientLogQueue
 
+processNotifications :: ClientState -> IO ()
+processNotifications st =
+  case notifyCmd (view (clientConfig . configNotifications) st) of
+    Just cmd | not (view clientUiFocused st) -> traverse_ (spawn cmd) (view clientNotifications st)
+    _ -> return ()
+  where
+    -- TODO: May be a nicer way to handle notification failure than just silently squashing the exception
+    handleException :: SomeException -> IO ()
+    handleException _ = return ()
+    spawn cmd pair = do
+      let procCfg = setStdin nullStream . setStdout nullStream . setStderr nullStream $ cmd pair
+      -- Maybe find a nicer way to get an error out of here.
+      catch (void (startProcess procCfg)) handleException
+
 -- | Respond to a network connection successfully connecting.
 doNetworkOpen ::
   Text        {- ^ network name -} ->
@@ -370,8 +387,7 @@ computeEffectiveTime time tags = fromMaybe time zncTime
 
 -- | Parses the time format used by ZNC for buffer playback
 parseZncTime :: String -> Maybe UTCTime
-parseZncTime = parseTimeM True defaultTimeLocale
-             $ iso8601DateFormat (Just "%T%Q%Z")
+parseZncTime = formatParseM iso8601Format
 
 
 -- | Update the height and width fields of the client state
@@ -401,6 +417,10 @@ doVtyEvent vty vtyEvent st =
     EvPaste utf8 ->
        do let str = Text.unpack (Text.decodeUtf8With Text.lenientDecode utf8)
           return $! Just $! over clientTextBox (Edit.insertPaste str) st
+    EvLostFocus ->
+      return (Just $! set clientUiFocused False st)
+    EvGainedFocus ->
+      return (Just $! set clientUiFocused True st)
     _ -> return (Just st)
 
 
