@@ -27,9 +27,9 @@ module Client.State.Focus
   , _Unfocused
   ) where
 
-import           Control.Lens
-import           Data.Text (Text)
-import           Irc.Identifier
+import Control.Lens (makePrisms)
+import Data.Text (Text)
+import Irc.Identifier (Identifier)
 
 -- | Currently focused window
 data Focus
@@ -45,7 +45,6 @@ data Subfocus
   = FocusMessages    -- ^ Show messages
   | FocusInfo        -- ^ Show channel metadata
   | FocusUsers       -- ^ Show channel user list
-  | FocusDCC         -- ^ Show DCC offers list
   | FocusMasks !Char -- ^ Show channel mask list for given mode
   | FocusWindows WindowsFilter -- ^ Show client windows
   | FocusPalette     -- ^ Show current palette
@@ -56,6 +55,8 @@ data Subfocus
   | FocusRtsStats    -- ^ Show GHC RTS statistics
   | FocusIgnoreList  -- ^ Show ignored masks
   | FocusCert        -- ^ Show rendered certificate
+  | FocusChanList (Maybe Int) (Maybe Int) -- ^ Show channel list
+  | FocusWho -- ^ Show last reply to a WHO query
   deriving (Eq,Show)
 
 -- | Unfocused first, followed by focuses sorted by network.
