@@ -27,6 +27,8 @@ module Client.State.Network
   -- * Lenses
   , csNick
   , csChannels
+  , csChannelList
+  , csWhoReply
   , csSocket
   , csModeTypes
   , csChannelTypes
@@ -49,6 +51,9 @@ module Client.State.Network
   , csAuthenticationState
   , csSeed
   , csAway
+  , clsElist
+  , clsDone
+  , clsItems
 
   -- * Cross-message state
   , Transaction(..)
@@ -80,46 +85,50 @@ module Client.State.Network
   , sendTopic
   ) where
 
-import qualified Client.Authentication.Ecdsa as Ecdsa
-import qualified Client.Authentication.Ecdh as Ecdh
-import qualified Client.Authentication.Scram as Scram
-import           Client.Configuration.ServerSettings
-import           Client.Network.Async
-import           Client.State.Channel
-import           Client.UserHost
-import           Client.Hook (MessageHook)
-import           Client.Hooks (messageHooks)
-import           Control.Lens
-import           Data.HashMap.Strict (HashMap)
-import qualified Data.HashMap.Strict as HashMap
-import qualified Data.HashSet as HashSet
-import qualified Data.ByteString as B
-import qualified Data.Map.Strict as Map
-import           Data.Bits
-import           Data.Foldable
-import           Data.List
-import qualified Data.List.NonEmpty as NonEmpty
-import           Data.Maybe
-import           Data.Text (Text)
-import qualified Data.Text as Text
-import qualified Data.Text.Encoding as Text
-import qualified Data.Text.Read as Text
-import           Data.Time
-import           Data.Time.Clock.POSIX
-import           Irc.Codes
-import           Irc.Commands
-import           Irc.Identifier
-import           Irc.Message
-import           Irc.Modes
-import           Irc.RawIrcMsg
-import           Irc.UserInfo
-import           LensUtils
-import qualified System.Random as Random
-import qualified Data.ByteString.Base64 as B64
+import Client.Authentication.Ecdh qualified as Ecdh
+import Client.Authentication.Ecdsa qualified as Ecdsa
+import Client.Authentication.Scram qualified as Scram
+import Client.Configuration.ServerSettings
+import Client.Hook (MessageHook)
+import Client.Hooks (messageHooks)
+import Client.Network.Async (abortConnection, send, NetworkConnection, TerminationReason(PingTimeout))
+import Client.State.Channel
+import Client.UserHost (UserAndHost(UserAndHost, _uhAccount))
+import Client.WhoReply
+import Control.Lens
+import Data.Bits (Bits((.&.)))
+import Data.ByteString qualified as B
+import Data.ByteString.Base64 qualified as B64
+import Data.Either qualified as Either
+import Data.Foldable (for_, traverse_ )
+import Data.HashMap.Strict (HashMap)
+import Data.HashMap.Strict qualified as HashMap
+import Data.HashSet qualified as HashSet
+import Data.List (foldl', delete, intersect, sort, sortBy, union)
+import Data.List.NonEmpty qualified as NonEmpty
+import Data.Map.Strict qualified as Map
+import Data.Maybe (fromMaybe, isJust, mapMaybe, listToMaybe)
+import Data.Text (Text)
+import Data.Text qualified as Text
+import Data.Text.Encoding qualified as Text
+import Data.Text.Read qualified as Text
+import Data.Time
+import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
+import Irc.Codes
+import Irc.Commands
+import Irc.Identifier (Identifier, idText, mkId)
+import Irc.Message
+import Irc.Modes
+import Irc.RawIrcMsg
+import Irc.UserInfo
+import LensUtils (overStrict, setStrict)
+import System.Random qualified as Random
 
 -- | State tracked for each IRC connection
 data NetworkState = NetworkState
   { _csChannels     :: !(HashMap Identifier ChannelState) -- ^ joined channels
+  , _csChannelList  :: !ChannelList -- ^ cached ELIST parameter and /list output
+  , _csWhoReply     :: !WhoReply -- ^ cached reply from the last WHO query
   , _csSocket       :: !NetworkConnection -- ^ network socket
   , _csModeTypes    :: !ModeTypes -- ^ channel mode meanings
   , _csUmodeTypes   :: !ModeTypes -- ^ user mode meanings
@@ -168,6 +177,13 @@ data PingStatus
   | PingConnecting !Int !(Maybe UTCTime) !ConnectRestriction -- ^ number of attempts, last known connection time
   deriving Show
 
+-- | Cached channel information from /list and elsewhere.
+data ChannelList = ChannelList
+  { _clsElist :: !(Maybe Text) -- ^ The last ELIST parameter used. Nothing is also used to trigger cache purges
+  , _clsDone  :: !Bool -- ^ Whether to purge the hash map on receiving a new RPL_LIST
+  , _clsItems :: ![(Identifier, Int, Text)] -- ^ The list of channel infos.
+  }
+
 data ConnectRestriction
   = NoRestriction       -- ^ no message restriction
   | StartTLSRestriction -- ^ STARTTLS hasn't finished
@@ -189,11 +205,25 @@ data Transaction
   | CapLsTransaction [(Text, Maybe Text)]
   deriving Show
 
+
 makeLenses ''NetworkState
+makeLenses ''ChannelList
 makePrisms ''Transaction
 makePrisms ''PingStatus
 makePrisms ''TimedAction
 
+newChannelList :: Maybe Text -> Maybe (Identifier, Int, Text) -> ChannelList
+newChannelList elist Nothing = ChannelList
+  { _clsElist = elist
+  , _clsDone = False
+  , _clsItems = []
+  }
+newChannelList elist (Just v) = ChannelList
+  { _clsElist = elist
+  , _clsDone = False
+  , _clsItems = [v]
+  }
+
 defaultChannelTypes :: String
 defaultChannelTypes = "#&"
 
@@ -271,6 +301,8 @@ newNetworkState ::
 newNetworkState network settings sock ping seed = NetworkState
   { _csUserInfo     = UserInfo "*" "" ""
   , _csChannels     = HashMap.empty
+  , _csChannelList  = newChannelList Nothing Nothing
+  , _csWhoReply     = finishWhoReply $ newWhoReply "" ""
   , _csSocket       = sock
   , _csChannelTypes = defaultChannelTypes
   , _csModeTypes    = defaultModeTypes
@@ -387,6 +419,7 @@ applyMessage' msgWhen msg cs =
         noReply (set (csUserInfo . uiHost) host cs)
 
     -- /who <#channel> %tuhna,616
+    -- TODO: Use a different magic token here?
     Reply _ RPL_WHOSPCRPL [_me,"616",user,host,nick,acct] ->
        let acct' = if acct == "0" then "*" else acct
        in noReply (recordUser (UserInfo (mkId nick) user host) acct' cs)
@@ -449,6 +482,26 @@ doRandomNick cs = reply [ircNick candidate] cs'
 
     (n, cs')    = cs & csSeed %%~ Random.randomR range
 
+doList :: [Text] -> NetworkState -> NetworkState
+doList (_:chan:users:topic) cs
+  | purge = set csChannelList (newChannelList elist (Just $! value)) cs
+  | otherwise = set (csChannelList . clsItems) items' cs
+  where
+    items' = value:(_clsItems . _csChannelList $ cs)
+    value = (mkId chan, usercount, fromMaybe "" (listToMaybe topic))
+    usercount = fst . Either.fromRight (0, "") . Text.decimal $ users
+    elist = _clsElist . _csChannelList $ cs
+    purge = _clsDone . _csChannelList $ cs
+doList _ cs = cs
+
+doListEnd :: NetworkState -> NetworkState
+doListEnd cs = set csChannelList ncl cs
+  where
+    ncl = ChannelList { _clsElist = elist, _clsDone = True, _clsItems = sorted }
+    elist = _clsElist . _csChannelList $ cs
+    sorted = sortBy sorter . _clsItems . _csChannelList $ cs
+    sorter (_, aU, _) (_, bU, _) = compare bU aU
+
 doTopic :: ZonedTime -> UserInfo -> Identifier -> Text -> NetworkState -> NetworkState
 doTopic when user chan topic =
   overChannel chan (setTopic topic . set chanTopicProvenance (Just $! prov))
@@ -583,14 +636,15 @@ doRpl cmd msgWhen args cs =
       case args of
         _me:_tgt:uname:host:_server:nick:_ ->
           noReply $
+          over csWhoReply (recordWhoReply args) $
           over csTransaction (\t ->
             let !x  = UserInfo (mkId nick) uname host
                 !xs = view _WhoTransaction t
             in WhoTransaction (x : xs))
             cs
         _ -> noReply cs
-
-    RPL_ENDOFWHO -> noReply (massRegistration cs)
+    RPL_WHOSPCRPL -> noReply (over csWhoReply (recordWhoXReply args) cs)
+    RPL_ENDOFWHO -> noReply (over csWhoReply finishWhoReply $ massRegistration cs)
 
     RPL_CHANNELMODEIS ->
       case args of
@@ -605,6 +659,10 @@ doRpl cmd msgWhen args cs =
     RPL_NOWAWAY -> noReply (set csAway True cs)
     RPL_UNAWAY  -> noReply (set csAway False cs)
 
+    -- /list
+    RPL_LIST    -> noReply (doList args cs)
+    RPL_LISTEND -> noReply (doListEnd cs)
+
     _ -> noReply cs
 
 
@@ -668,6 +726,9 @@ squelchReply rpl =
     RPL_CREATIONTIME    -> True
     RPL_CHANNEL_URL     -> True
     RPL_NOTOPIC         -> True
+    RPL_LISTSTART       -> True
+    RPL_LIST            -> True
+    RPL_LISTEND         -> True
     _                   -> False
 
 -- | Return 'True' for messages that should be hidden outside of
@@ -768,7 +829,7 @@ selectCaps cs offered = (supported `intersect` Map.keys capMap)
       ["multi-prefix", "batch", "znc.in/playback", "znc.in/self-message"
       , "cap-notify", "extended-join", "account-notify", "chghost"
       , "userhost-in-names", "account-tag", "solanum.chat/identify-msg"
-      , "solanum.chat/realhost" ]
+      , "solanum.chat/realhost", "away-notify"]
 
     -- logic for using IRCv3.2 server-time if available and falling back
     -- to ZNC's specific extension otherwise.
